--- gnus-art.el	1999/07/01 08:44:50	1.1
+++ gnus-art.el	1999/07/01 09:02:06
@@ -2337,6 +2337,36 @@
 	 (concat (gnus-newsgroup-directory-form newsgroup) "/news"))
        gnus-article-save-directory)))
 
+(defun article-restore-broken-multipart nil
+  "Restore broken multipart."
+  (interactive)
+  (let (boundary pt ihandles buffer-read-only)
+    (with-current-buffer gnus-original-article-buffer
+      (article-goto-body)
+      (setq pt (point))
+      (while (and (not boundary)
+		  (re-search-forward "^--\\(.*[^ ]\\) *$" nil t))
+	(setq boundary (match-string 1))
+	(forward-line)
+	    (unless (re-search-forward (concat "^--" (regexp-quote boundary) 
+					       "\\(--\\)? *$") nil t)
+	      (setq boundary nil))))
+    (when boundary
+      (article-goto-body)
+      (save-restriction
+	(narrow-to-region (point) (point-max))
+	(delete-region (point) (point-max))
+	(insert-buffer-substring
+	 gnus-original-article-buffer pt)
+	(setq ihandles 
+	      (cons "multipart/mixed"
+		    (mm-dissect-multipart 
+			   (list "multipart/mixed" 
+				 (cons 'boundary boundary)))))
+	(delete-region (point-min) (point-max)))
+      (gnus-display-mime ihandles)
+      (set (make-local-variable 'gnus-article-mime-handles) ihandles))))
+
 (eval-and-compile
   (mapcar
    (lambda (func)
@@ -2391,6 +2421,7 @@
      article-emphasize
      article-treat-dumbquotes
      article-normalize-headers
+     article-restore-broken-multipart
      (article-show-all . gnus-article-show-all-headers))))
 
 ;;;
--- gnus-sum.el	1999/07/01 08:47:45	1.2
+++ gnus-sum.el	1999/07/01 08:51:26
@@ -1518,6 +1518,7 @@
     "w" gnus-article-decode-mime-words
     "c" gnus-article-decode-charset
     "v" gnus-mime-view-all-parts
+    "r" gnus-article-restore-broken-multipart
     "b" gnus-article-view-part)
 
   (gnus-define-keys (gnus-summary-wash-time-map "T" gnus-summary-wash-map)
@@ -1637,6 +1638,7 @@
 	      ["Words" gnus-article-decode-mime-words t]
 	      ["Charset" gnus-article-decode-charset t]
 	      ["QP" gnus-article-de-quoted-unreadable t]
+	      ["Broken multipart" gnus-article-restore-broken-multipart t]
 	      ["View all" gnus-mime-view-all-parts t])
              ("Date"
               ["Local" gnus-article-date-local t]
